@@ -41,13 +41,23 @@ class Gateway(gateway_base.BaseGateway):
         self._group._unregister(self)
         self._trace("--> sending GATEWAY_TERMINATE")
         try:
-            self._send(Message.GATEWAY_TERMINATE(0, ''))
+            self._send(Message.GATEWAY_TERMINATE)
             self._io.close_write()
         except IOError:
             v = sys.exc_info()[1]
             self._trace("io-error: could not send termination sequence")
             self._trace(" exception: %r" % v)
 
+    def reconfigure(self, py2str_as_py3str=True, py3str_as_py2str=False):
+        """
+        set the string coercion for this gateway
+        the default is to try to convert py2 str as py3 str,
+        but not to try and convert py3 str to py2 str
+        """
+        self._unserializer.py2str_as_py3str = py2str_as_py3str
+        self._unserializer.py3str_as_py2str = py3str_as_py2str
+        self._send(Message.RECONFIGURE, data=(py2str_as_py3str, py3str_as_py2str))
+
     def _remote_bootstrap_gateway(self, io):
         """ send gateway bootstrap code to a remote Python interpreter
             endpoint, which reads from io for a string to execute.
@@ -75,7 +85,7 @@ class Gateway(gateway_base.BaseGateway):
     def remote_status(self):
         """ return information object about remote execution status. """
         channel = self.newchannel()
-        self._send(Message.STATUS(channel.id))
+        self._send(Message.STATUS, channel.id)
         statusdict = channel.receive()
         # the other side didn't actually instantiate a channel
         # so we just delete the internal id/channel mapping
@@ -112,8 +122,9 @@ class Gateway(gateway_base.BaseGateway):
             raise TypeError("can't pass kwargs to non-function remote_exec")
 
         channel = self.newchannel()
-        self._send(Message.CHANNEL_EXEC(channel.id,
-                                       (source, call_name, kwargs)))
+        self._send(Message.CHANNEL_EXEC,
+                   channel.id,
+                   (source, call_name, kwargs))
         return channel
 
     def remote_init_threads(self, num=None):
