@@ -102,82 +102,75 @@ class Popen2IO:
 
 class Message:
     """ encapsulates Messages and their wire protocol. """
-    _types = {}
+    _types = []
 
-    def __init__(self, channelid=0, data=''):
+    def __init__(self, msgcode, channelid=0, data=''):
+        self.msgcode = msgcode
         self.channelid = channelid
         self.data = data
 
-    def writeto(self, io):
-        serialize(io, (self.msgtype, self.channelid, self.data))
-
-    def readfrom(cls, io, channelfactory):
-        unserializer = Unserializer(io, channelfactory)
-        msgtype, senderid, data = unserializer.load()
-        return cls._types[msgtype](senderid, data)
-
-    readfrom = classmethod(readfrom)
+    def received(self, gateway):
+        self._types[self.msgcode](self, gateway)
 
     def __repr__(self):
+        name = self._types[self.msgcode].__name__.upper()
         r = repr(self.data)
         if len(r) > 50:
-            return "<Message.%s channelid=%d len=%d>" %(self.__class__.__name__,
+            return "<Message.%s channelid=%d len=%d>" %(name,
                         self.channelid, len(r))
         else:
-            return "<Message.%s channelid=%d %r>" %(self.__class__.__name__,
+            return "<Message.%s channelid=%d %s>" %(name,
                         self.channelid, self.data)
 
 def _setupmessages():
-    class STATUS(Message):
-        def received(self, gateway):
-            # we use self.channelid to send back information
-            # but don't instantiate a channel object
-            active_channels = gateway._channelfactory.channels()
-            numexec = 0
-            for ch in active_channels:
-                if getattr(ch, '_executing', False):
-                    numexec += 1
-            d = {'execqsize': gateway._execqueue.qsize(),
-                 'numchannels': len(active_channels),
-                 'numexecuting': numexec
-            }
-            gateway._send(Message.CHANNEL_DATA(self.channelid, d))
+    def status(message, gateway):
+        # we use the channelid to send back information
+        # but don't instantiate a channel object
+        active_channels = gateway._channelfactory.channels()
+        numexec = 0
+        for ch in active_channels:
+            if getattr(ch, '_executing', False):
+                numexec += 1
+        d = {'execqsize': gateway._execqueue.qsize(),
+             'numchannels': len(active_channels),
+             'numexecuting': numexec
+        }
+        gateway._send(Message.CHANNEL_DATA, message.channelid, d)
 
-    class CHANNEL_EXEC(Message):
-        def received(self, gateway):
-            channel = gateway._channelfactory.new(self.channelid)
-            gateway._local_schedulexec(channel=channel, sourcetask=self.data)
+    def channel_exec(message, gateway):
+        channel = gateway._channelfactory.new(message.channelid)
+        gateway._local_schedulexec(channel=channel, sourcetask=message.data)
 
-    class CHANNEL_DATA(Message):
-        def received(self, gateway):
-            gateway._channelfactory._local_receive(self.channelid, self.data)
+    def channel_data(message, gateway):
+        gateway._channelfactory._local_receive(message.channelid, message.data)
 
-    class CHANNEL_CLOSE(Message):
-        def received(self, gateway):
-            gateway._channelfactory._local_close(self.channelid)
+    def channel_close(message, gateway):
+        gateway._channelfactory._local_close(message.channelid)
 
-    class CHANNEL_CLOSE_ERROR(Message):
-        def received(self, gateway):
-            remote_error = RemoteError(self.data)
-            gateway._channelfactory._local_close(self.channelid, remote_error)
+    def channel_close_error(message, gateway):
+        remote_error = RemoteError(message.data)
+        gateway._channelfactory._local_close(message.channelid, remote_error)
 
-    class CHANNEL_LAST_MESSAGE(Message):
-        def received(self, gateway):
-            gateway._channelfactory._local_close(self.channelid, sendonly=True)
+    def channel_last_message(message, gateway):
+        gateway._channelfactory._local_close(message.channelid, sendonly=True)
 
-    class GATEWAY_TERMINATE(Message):
-        def received(self, gateway):
-            gateway._terminate_execution()
-            raise SystemExit(0)
+    def gateway_terminate(message, gateway):
+        gateway._terminate_execution()
+        raise SystemExit(0)
 
-    classes = [
-        STATUS, CHANNEL_EXEC, CHANNEL_DATA, CHANNEL_CLOSE,
-        CHANNEL_CLOSE_ERROR, CHANNEL_LAST_MESSAGE, GATEWAY_TERMINATE
+    def reconfigure(message, gateway):
+        py2str_as_py3str, py3str_as_py2str = message.data
+        gateway._unserializer.py2str_as_py3str = py2str_as_py3str
+        gateway._unserializer.py3str_as_py2str = py3str_as_py2str
+
+    types = [
+        status, reconfigure, gateway_terminate,
+        channel_exec, channel_data, channel_close,
+        channel_close_error, channel_last_message,
     ]
-    for i, cls in enumerate(classes):
-        Message._types[i] = cls
-        cls.msgtype = i
-        setattr(Message, cls.__name__, cls)
+    for i, handler in enumerate(types):
+        Message._types.append(handler)
+        setattr(Message, handler.__name__.upper(), i)
 
 _setupmessages()
 
@@ -291,11 +284,11 @@ class Channel(object):
         else:
             # state transition "opened" --> "deleted"
             if self._items is None:    # has_callback
-                Msg = Message.CHANNEL_LAST_MESSAGE
+                msgcode = Message.CHANNEL_LAST_MESSAGE
             else:
-                Msg = Message.CHANNEL_CLOSE
+                msgcode = Message.CHANNEL_CLOSE
             try:
-                self.gateway._send(Msg(self.id))
+                self.gateway._send(msgcode, self.id)
             except (IOError, ValueError): # ignore problems with sending
                 pass
 
@@ -347,9 +340,9 @@ class Channel(object):
             if not self._receiveclosed.isSet():
                 put = self.gateway._send
                 if error is not None:
-                    put(Message.CHANNEL_CLOSE_ERROR(self.id, error))
+                    put(Message.CHANNEL_CLOSE_ERROR, self.id, error)
                 else:
-                    put(Message.CHANNEL_CLOSE(self.id))
+                    put(Message.CHANNEL_CLOSE, self.id)
                 self._trace("sent channel close message")
             if isinstance(error, RemoteError):
                 self._remoteerrors.append(error)
@@ -387,8 +380,7 @@ class Channel(object):
         """
         if self.isclosed():
             raise IOError("cannot send to %r" %(self,))
-        data = Message.CHANNEL_DATA(self.id, item)
-        self.gateway._send(data)
+        self.gateway._send(Message.CHANNEL_DATA, self.id, item)
 
     def receive(self, timeout=-1):
         """receive a data item that was sent from the other side.
@@ -520,7 +512,7 @@ class ChannelFactory(object):
                 excinfo = sys.exc_info()
                 self.gateway._trace("exception during callback: %s" % excinfo[1])
                 errortext = self.gateway._geterrortext(excinfo)
-                self.gateway._send(Message.CHANNEL_CLOSE_ERROR(id, errortext))
+                self.gateway._send(Message.CHANNEL_CLOSE_ERROR, id, errortext)
                 self._local_close(id, errortext)
 
     def _finished_receiving(self):
@@ -539,6 +531,9 @@ class ChannelFile(object):
         self.channel = channel
         self._proxyclose = proxyclose
 
+    def isatty(self):
+        return False
+
     def close(self):
         if self._proxyclose:
             self.channel.close()
@@ -594,6 +589,7 @@ class BaseGateway(object):
         self._io = io
         self.id = id
         self._channelfactory = ChannelFactory(self, _startcount)
+        self._unserializer = Unserializer(self._io, self._channelfactory)
         self._receivelock = threading.RLock()
         # globals may be NONE at process-termination
         self._trace = trace
@@ -614,7 +610,7 @@ class BaseGateway(object):
         try:
             try:
                 while 1:
-                    msg = Message.readfrom(self._io, self._channelfactory)
+                    msg = Message(*self._unserializer.load())
                     self._trace("received", msg)
                     _receivelock = self._receivelock
                     _receivelock.acquire()
@@ -647,10 +643,9 @@ class BaseGateway(object):
     def _terminate_execution(self):
         pass
 
-    def _send(self, msg):
-        assert isinstance(msg, Message)
-        msg.writeto(self._io)
-        self._trace('sent', msg)
+    def _send(self, msgcode, channelid=0, data=''):
+        serialize(self._io, (msgcode, channelid, data))
+        self._trace('sent', Message(msgcode, channelid, data))
 
     def _local_schedulexec(self, channel, sourcetask):
         channel.close("execution disallowed")
@@ -769,10 +764,10 @@ class UnserializationError(SerializeError):
     """Error while unserializing an object."""
 
 if ISPY3:
-    def b(s):
-        return s.encode("latin-1")
+    def bchr(n):
+        return bytes([n])
 else:
-    b = str
+    bchr = chr
 
 FOUR_BYTE_INT_MAX = 2147483647
 
@@ -889,23 +884,23 @@ class Unserializer(object):
     def load_newdict(self):
         self.stack.append({})
 
-    def _load_tuple(self):
+    def _load_collection(self, type_):
         length = self._read_int4()
         if length:
-            tup = tuple(self.stack[-length:])
+            res = type_(self.stack[-length:])
             del self.stack[-length:]
+            self.stack.append(res)
         else:
-            tup = ()
-        return tup
+            self.stack.append(type_())
 
     def load_buildtuple(self):
-        self.stack.append(self._load_tuple())
+        self._load_collection(tuple)
 
     def load_set(self):
-        self.stack.append(set(self._load_tuple()))
+        self._load_collection(set)
 
     def load_frozenset(self):
-        self.stack.append(frozenset(self._load_tuple()))
+        self._load_collection(frozenset)
 
     def load_stop(self):
         raise _Stop
@@ -929,7 +924,7 @@ def _buildopcodes():
     l.sort()
     for i,(opname, func) in enumerate(l):
         assert i < 26, "xxx"
-        i = b(chr(64+i))
+        i = bchr(64+i)
         Unserializer.num2func[i] = func
         setattr(opcode, opname, i)
 
