@@ -3,7 +3,8 @@ import py
 import sys, os, subprocess, inspect
 import execnet
 from execnet import gateway_base, gateway
-from execnet.gateway_base import Message, Channel, ChannelFactory
+from execnet.gateway_base import Message, Channel, ChannelFactory, serialize, \
+        Unserializer
 
 def test_errors_on_execnet():
     assert hasattr(execnet, 'RemoteError')
@@ -67,20 +68,21 @@ def test_io_message(anypython, tmpdir):
         temp_in = BytesIO()
         io = Popen2IO(temp_out, temp_in)
         unserializer = Unserializer(io)
-        for i, msg_cls in Message._types.items():
-            print ("checking %s %s" %(i, msg_cls))
+        for i, handler in enumerate(Message._types):
+            print ("checking %s %s" %(i, handler))
             for data in "hello", "hello".encode('ascii'):
-                msg1 = msg_cls(i, data)
-                msg1.writeto(io)
+                msg1 = Message(i, i, data)
+                serialize(io, (i, i, data))
                 x = io.outfile.getvalue()
                 io.outfile.truncate(0)
                 io.outfile.seek(0)
                 io.infile.seek(0)
                 io.infile.write(x)
                 io.infile.seek(0)
-                msg2 = Message.readfrom(io, None)
+                msg2 = Message(*unserializer.load())
                 assert msg1.channelid == msg2.channelid, (msg1, msg2)
                 assert msg1.data == msg2.data
+                assert msg1.msgcode == msg2.msgcode
         print ("all passed")
     """))
     #out = py.process.cmdexec("%s %s" %(executable,check))
@@ -173,13 +175,14 @@ def test_exectask():
 
 class TestMessage:
     def test_wire_protocol(self):
-        for cls in Message._types.values():
+        for i, handler in enumerate(Message._types):
             one = py.io.BytesIO()
             data = '23'.encode('ascii')
-            cls(42, data).writeto(one)
+            serialize(one, (i, 42, data))
             two = py.io.BytesIO(one.getvalue())
-            msg = Message.readfrom(two, None)
-            assert isinstance(msg, cls)
+            msg = Message(*Unserializer(two, None).load())
+            assert msg.msgcode == i
+            assert isinstance(msg, Message)
             assert msg.channelid == 42
             assert msg.data == data
             assert isinstance(repr(msg), str)
