@@ -5,7 +5,7 @@ import os, sys, time
 import py
 import execnet
 from execnet import gateway_base, gateway
-
+from testing.test_serializer import _find_version
 TESTTIMEOUT = 10.0 # seconds
 needs_osdup = py.test.mark.skipif("not hasattr(os, 'dup')")
 
@@ -356,3 +356,40 @@ class TestTracing:
     def test_no_tracing_by_default(self):
         assert gateway_base.trace == gateway_base.notrace, \
                 "trace does not to default to empty tracing"
+
+class TestStringCoerce:
+    @py.test.mark.skipif('sys.version>="3.0"')
+    def test_2to3(self):
+        python = _find_version('3')
+        gw = execnet.makegateway('popen//python=%s'%python)
+        ch = gw.remote_exec('channel.send(channel.receive())')
+        ch.send('a')
+        res = ch.receive()
+        assert isinstance(res, unicode)
+
+        gw.reconfigure(py3str_as_py2str=True)
+
+        ch = gw.remote_exec('channel.send(channel.receive())')
+        ch.send('a')
+        res = ch.receive()
+        assert isinstance(res, str)
+        gw.exit()
+
+    @py.test.mark.skipif('sys.version<"3.0"')
+    def test_3to2(self):
+        python = _find_version('2')
+        gw = execnet.makegateway('popen//python=%s'%python)
+
+        ch = gw.remote_exec('channel.send(channel.receive())')
+        ch.send(bytes('a', 'ascii'))
+        res = ch.receive()
+        assert isinstance(res, str)
+
+        gw.reconfigure(py3str_as_py2str=True, py2str_as_py3str=False)
+
+        ch = gw.remote_exec('channel.send(channel.receive())')
+        ch.send('a')
+        res = ch.receive()
+        assert isinstance(res, bytes)
+        gw.exit()
+
