@@ -12,6 +12,16 @@ def _find_version(suffix=""):
     name = "python" + suffix
     executable = py.path.local.sysfind(name)
     if executable is None:
+        if suffix == "2":
+            for name in ('python2.6', 'python2.7'):
+                executable = py.path.local.sysfind(name)
+                if executable:
+                    return executable
+        elif suffix == "3":
+            for name in ('python3.1', 'python3.2'):
+                executable = py.path.local.sysfind(name)
+                if executable:
+                    return executable
         if sys.platform == "win32" and suffix == "3":
             for name in ('python31', 'python30'):
                 executable = py.path.local(r"c:\\%s\python.exe" % (name,))
@@ -24,7 +34,7 @@ def setup_module(mod):
     mod.TEMPDIR = py.path.local(tempfile.mkdtemp())
     if sys.version_info > (3, 0):
         mod._py3_wrapper = PythonWrapper(py.path.local(sys.executable))
-        mod._py2_wrapper = PythonWrapper(_find_version())
+        mod._py2_wrapper = PythonWrapper(_find_version("2"))
     else:
         mod._py3_wrapper = PythonWrapper(_find_version("3"))
         mod._py2_wrapper = PythonWrapper(py.path.local(sys.executable))
@@ -46,8 +56,8 @@ sys.path.insert(0, %r)
 from execnet import gateway_base as serializer
 if sys.version_info > (3, 0): # Need binary output
     sys.stdout = sys.stdout.detach()
-saver = serializer.Serializer(sys.stdout)
-saver.save(%s)""" % (pyimportdir, obj_rep,))
+saver = serializer.serialize(sys.stdout, %s)
+""" % (pyimportdir, obj_rep,))
         popen = subprocess.Popen([str(self.executable), str(script_file)],
                                  stdin=subprocess.PIPE,
                                  stderr=subprocess.PIPE,
@@ -212,11 +222,11 @@ def test_str(py2, py3):
     assert tp == "str"
     assert s == "'xyz'"
     tp, s = py3.load(p, "py2str_as_py3str=True")
-    assert tp == "str" 
+    assert tp == "str"
     assert s == "'xyz'"
     tp, s = py3.load(p, "py2str_as_py3str=False")
     assert s == "b'xyz'"
-    assert tp == "bytes" 
+    assert tp == "bytes"
 
 def test_unicode(py2, py3):
     p = py2.dump("u'hi'")
