@@ -73,10 +73,13 @@ def test_close_initiating_remote_no_error(testdir, anypython):
     """ % str(execnetdir))
     popen = subprocess.Popen([str(anypython), str(p)],
             stdout=subprocess.PIPE, stderr=subprocess.PIPE,)
-    stdout, stderr = popen.communicate()
-    print (stdout)
-    print (stderr)
-    assert not stderr
+    out, err = popen.communicate()
+    print (out)
+    print (err)
+    err = err.decode('utf8')
+    lines = [x for x in err.splitlines()
+               if '*sys-package' not in x]
+    assert not lines
 
 def test_terminate_implicit_does_trykill(testdir, anypython, capfd):
     p = testdir.makepyfile("""
@@ -98,4 +101,6 @@ def test_terminate_implicit_does_trykill(testdir, anypython, capfd):
     reply = WorkerPool(1).dispatch(popen.communicate)
     reply.get(timeout=50)
     out, err = capfd.readouterr()
-    assert not err or "Killed" in err
+    lines = [x for x in err.splitlines()
+               if '*sys-package' not in x]
+    assert not lines or "Killed" in err
